#!/usr/bin/env python

#
# ===== NW Configurator UI v.0.1 =====
#
# This is simple program that configures the
# == New Wave pixmap theme == 
# in an easy way.
# 
# Anton Kerezov <ankere@gmail.com>
# License GPL v3.0

import sys
import os
import user
import pygtk
pygtk.require("2.0")
import gtk
import nw_core


class ThemePickerUI:
    
    ThemeDir = ""
     
    def on_dlgPicker_close(self, widget, data=None):
        ThemeDir = ""
        
    
    def on_btnCancel_clicked(self, widget, data=None):
        ThemeDir = ""
        
        
    def on_btnEdit_clicked(self, widget, data=None):
        model = self.combo.get_model()
        active = self.combo.get_active()
        if active < 0:
            self.ThemeDir = ""
        self.ThemeDir =  model[active][0]
    
    
    def __init__(self, dirs):

        builder_pick = gtk.Builder()
        builder_pick.add_from_file("nw_conf.glade") 

        self.dialog = builder_pick.get_object("dlgPicker")
        self.combo = builder_pick.get_object("cmbLocations")
        #self.lstItems = builder.get_object("lstItems")
        
        for path in dirs:
            self.combo.append_text(path) 
        
        #make the first choise actve should be the home 
        #dir if available or the root one    
        self.combo.set_active(0)
        
        builder_pick.connect_signals(self)
        
        
    def showdialog(self):
        self.dialog.run()
        self.dialog.destroy()
        
        return self.ThemeDir
        


class NewWaveConfiguratorUI:
    
    NoFile = False          # If there is no file specifide this is changed to True
    RequiredThemeVer = "0.8" # Specify for a major version 0.8 or 1.6 for example 
                            #      and miss the minor ones unless you want a very 
                            #      specific configurator for  0.8.3 for example
    rc = None               # The main class to handle theme file
    HasChanges = False      # Marks if there are changes to the UI
    HasSaves = False        # If the file has been saved at least once
    Reverting = False       # If the user has pressed the Rever buton or on file load
    IsDarkVariant = False   # If the theme loaded is with dark menus
    RestartDialog = True    # If the succes save dialog shows eachtime you save [disabled]
    
    Debug = False           # Internal variable that enables more info to the terminal output.
                            #      Set to True to see them Errors are always printed.
    
    
    def on_frmMain_destroy(self, widget, data=None):
        if self.ask_save() == True:
            gtk.main_quit()
        
            
    def on_mnu_file_quit_activate(self, widget, data=None):
        if self.ask_save() == True:
            gtk.main_quit()
            
        
    def on_mnu_help_about_activate(self, widget, data=None):
        if self.Debug:
            print "About dialog"
            
        builder_a = gtk.Builder()
        builder_a.add_from_file("nw_conf.glade") 

        self.aboutdialog = builder_a.get_object("dlgAbout")
        builder_a.connect_signals(self)
        
        self.aboutdialog.run()
        self.aboutdialog.destroy()
            
            
    def on_save_theme_file_activate(self, widget, data=None):
        if self.save_file() == True:
            #show this message only once
            #if self.RestartDialog == True:
            dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_INFO, buttons= gtk.BUTTONS_OK, message_format="Changes are saved to the theme.\nTo see them you must reset it form the Appearance dialog.")
            dlg.set_title("File Saved")            
            dlg.run()
            dlg.destroy()
            #self.RestartDialog == False
        else:
            errdlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons= gtk.BUTTONS_OK, message_format="Save Failed. Check if the file has not been moved or renamed.")
            errdlg.set_title("Save Error")
            errdlg.run()
            errdlg.destroy()
    

    def on_revert_changes_activate(self, button, data=None):
        self.Reverting = True
        self.rc.restore_file_data()
        
        if self.HasSaves == False:
            self.HasChanges = False
        else: 
            self.HasChanges = True
            
        self.__update_ui()
        self.Reverting = False
        
        if self.Debug:
            print "Reverted to first loaded."
            
    
    def on_mnu_file_info_activate(self, button, data=None):
        ver = self.rc.get_version_string(self.RequiredThemeVer)
        
        dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_INFO, buttons= gtk.BUTTONS_OK, message_format="Full file path:\n" +"'" + self.rc.get_filename()+"'" + "\n\n" + ver)
        dlg.set_title("File Info")
        dlg.run()
        dlg.destroy()


    def on_rbTopBottom_toggled(self, button, data=None):
        if not self.Reverting:
            if self.StepBoth.get_active(): #Both arrow activated
                if self.modify_scroll(1, 1, 0, 0):
                    self.HasChanges = True
                else:
                    print "Failed to change steppers"
                
        
    def on_rbBottom_toggled(self, button, data=None):
        if not self.Reverting:
            if self.StepBottom.get_active(): 
                if self.modify_scroll(0, 1, 1, 0):
                    self.HasChanges = True
                else:
                    print "Failed to change steppers"
        

    def on_rbTop_toggled(self, button, data=None):
        if not self.Reverting:
            if self.StepTop.get_active():
                if self.modify_scroll(1, 0, 0, 1):
                    self.HasChanges = True
                else:
                    print "Failed to change steppers"


    def on_rbNone_toggled(self, button, data=None):
        if not self.Reverting:
            if self.StepNone.get_active(): # No steppers
                if self.modify_scroll(0, 0, 0, 0):
                    self.HasChanges = True
                else:
                    print "Failed to change steppers"
        
    
    def modify_scroll(self, down, up, sdown, sup):
        """A complex method for changing scrollbars according to switchs
        """
        bStep = int(self.rc.get_property_value("::has-backward-stepper"))
        fStep = int(self.rc.get_property_value("::has-forward-stepper"))
        sbStep = int(self.rc.get_property_value("::has-secondary-backward-stepper"))
        sfStep = int(self.rc.get_property_value("::has-secondary-forward-stepper"))
        
        bStep_old = "GtkScrollbar::has-backward-stepper=" + str(bStep)
        bStep_new = "\tGtkScrollbar   ::has-backward-stepper = " + str(up)
        
        fStep_old = "GtkScrollbar::has-forward-stepper=" + str(fStep)
        fStep_new = "\tGtkScrollbar   ::has-forward-stepper = " + str(down)
        
        sbStep_old = "GtkScrollbar::has-secondary-backward-stepper=" + str(sbStep)
        sbStep_new = "\tGtkScrollbar   ::has-secondary-backward-stepper = " + str(sup)
        
        sfStep_old = "GtkScrollbar::has-secondary-forward-stepper=" + str(sfStep)
        sfStep_new = "\tGtkScrollbar   ::has-secondary-forward-stepper = " + str(sdown)
        
        Transaction = True
        
        if not bStep == up:
            if not self.rc.modify_line(bStep_old,bStep_new):
                    Transaction = False
        if not fStep == down:
            if not self.rc.modify_line(fStep_old,fStep_new):
                    Transaction = False
        if not sbStep == sup:
            if not self.rc.modify_line(sbStep_old,sbStep_new):
                    Transaction = False
        if not sfStep == sdown:
            if not self.rc.modify_line(sfStep_old,sfStep_new):
                    Transaction = False
                    
        if Transaction:
            if self.Debug:
                print "Scrollbars modified."
            return True
        else:
            return False
            
    
    def on_chkFFFix_toggled(self, check, data=None):
        if not self.Reverting:
            Line = self.rc.find_line("widget_class \"*Fix*MenuBar*\"")
            
            if self.Debug:
                print "Old line: " + Line

            if self.FixCheck.get_active() == True:
                if self.rc.comment_line(Line, False):
                    self.HasChanges = True
            else:
                if self.rc.comment_line(Line, True):
                    self.HasChanges = True
     
            if self.Debug:
                Line = self.rc.find_line("widget_class \"*Fix*MenuBar*\"")
                print "New line: " + Line


    def on_adjTootltip_value_changed(self, spin, data=None):
        if not self.Reverting:
            old = "gtk-tooltip-timeout=" + self.rc.get_property_value("gtk-tooltip-timeout")
            new = "gtk-tooltip-timeout = " + str(self.tooltipspin.get_value_as_int())
        
            if self.rc.modify_line(old,new):
                self.HasChanges = True
            
            if self.Debug:
                print "Spin chnaged"
                print old
                print new + "\n"
        

    def on_adjTaskOverTransp_value_changed(self, pgrogress, data=None):
        if not self.Reverting:
            old = "WnckTasklist::fade-opacity=" + self.rc.get_property_value("::fade-opacity")
            new = "\tWnckTasklist   ::fade-opacity\t        = " + str(self.taskO.get_value()/100)
            
            if self.rc.modify_line(old,new):
                self.HasChanges = True
            
            if self.Debug:
                print "Taskbar transparency changed:"
                print old
                print new + "\n"
                
        
    def on_adjDeskTransp_value_changed(self, pgrogress, data=None):
        if not self.Reverting:
            old = "NautilusIconContainer::highlight_alpha=" + self.rc.get_property_value("NautilusIconContainer::highlight_alpha")
            new = "\tNautilusIconContainer::highlight_alpha= " + str(int(self.deskO.get_value()*2.55))
            
            if self.rc.modify_line(old,new):
                self.HasChanges = True
            
            if self.Debug:    
                print "Desktop transparency changed"
                print old
                print new + "\n"

        
    def __init__(self, path):
        if path == '':
            self.NoFile = True
        
        
        builder = gtk.Builder()
        builder.add_from_file("nw_conf.glade") 
        self.window = builder.get_object("frmMain")
        
        #get the ref to the controls
        #tooltip spin
        self.tooltipspin = builder.get_object("spinTooltipTimeout")
        
        #range controls for opacity
        self.deskO = builder.get_object("rngTransparency")
        self.taskO = builder.get_object("rngTaskTransp")
        
        #Option buttons
        self.StepBoth = builder.get_object("rbTopBottom")
        self.StepBottom = builder.get_object("rbBottom")
        self.StepTop = builder.get_object("rbTop")
        self.StepNone = builder.get_object("rbNone")
        
        #the FF fix objects
        self.FixBox = builder.get_object("vboxFix")
        self.FixCheck = builder.get_object("chkFFFix")


        builder.connect_signals(self)
        
        if path.find("Dark") != -1:
            self.IsDarkVariant = True
    
        self.rc = nw_core.GtkRcFile(path)
        if self.rc.load_file():
            #rc.print_contents()
            
            if not self.rc.theme_version(self.RequiredThemeVer):
                print "Theme not compatible with configurator. Required theme version: " + self.RequiredThemeVer + ".*\n\nPleae select another theme."
                errdlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons= gtk.BUTTONS_OK, message_format="Theme not compatible with configurator.\nRequired theme version: " + self.RequiredThemeVer + ".*")
                errdlg.set_title("Load Error")
                errdlg.run()
                errdlg.destroy()
                self.NoFile = True
            else:
                if not self.IsDarkVariant:
                    self.FixBox.show()
                
                self.__update_ui()

                
        else:
            print "Could not load file"
            self.NoFile = True
            
        


    def __update_ui(self):
        # This is used in order for the UI not to recieve event for the updates
        self.Reverting = True
        
        # Update the tooltip value
        val = self.rc.get_property_value("gtk-tooltip-timeout")
        if val == '' or val == None:
            val = "850"
        defSpin = float(val)
        self.tooltipspin.set_value(defSpin) 
   
        # Error values handling
        val2 = self.rc.get_property_value("NautilusIconContainer::highlight_alpha")
        if val2 == '' or val2 == None:
            val2 = "145"
        # Because the max is 255 1% is 2.55 so we devide the value to get the approx %
        defDeskO = float(val2) / 2.55 
        self.deskO.set_value(defDeskO) 
        
        # Max here is 1.0 so we transform it to 100 to fit our scale
        val3 = self.rc.get_property_value("::fade-opacity")
        if val3 == '' or val3 == None:
            val3 = "0.6"
        defTaskO = 100 * float(val3) 
        self.taskO.set_value(defTaskO)
    
        # Set the option btns for scrollbar arrows and check the values beforehand
        val_missing = False # if one line does not have valuse stop loading others too
        bs = self.rc.get_property_value("::has-backward-stepper")
        if bs == '':
            bs = "1"
            val_missing = True
        bStep = int(bs)
        
        fs = self.rc.get_property_value("::has-forward-stepper")
        if fs == '' or val_missing:
            val_missing = True
            fs = "1"
        fStep = int(fs)
        
        sbs = self.rc.get_property_value("::has-secondary-backward-stepper")
        if sbs == "" or val_missing:
            val_missing = True
            sbs = "0"
        sbStep = int(sbs)
        
        sfs = self.rc.get_property_value("::has-secondary-forward-stepper")
        if sfs == '' or val_missing:
            sfs = '0'
        sfStep = int(sfs)
        
        # Update the option buttons
        if bStep == 1 and fStep == 1 and sbStep == 0 and sfStep == 0:
            self.StepBoth.set_active(True)
            self.StepBottom.set_active(False)
            self.StepTop.set_active(False)
            self.StepNone.set_active(False)
        elif bStep == 0 and fStep == 0 and sbStep == 0 and sfStep == 0:
            self.StepBoth.set_active(False)
            self.StepBottom.set_active(False)
            self.StepTop.set_active(False)
            self.StepNone.set_active(True)
        elif bStep == 1 and fStep == 0 and sbStep == 0 and sfStep == 1: 
            self.StepBoth.set_active(False)
            self.StepBottom.set_active(False)
            self.StepTop.set_active(True)
            self.StepNone.set_active(False)
        elif bStep == 0 and fStep == 1 and sbStep == 1 and sfStep == 0: 
            self.StepBoth.set_active(False)
            self.StepBottom.set_active(True)
            self.StepTop.set_active(False)
            self.StepNone.set_active(False)
            
        # Update the fix checkbox
        if not self.IsDarkVariant:
            Line = self.rc.find_line("widget_class \"*Fix*MenuBar*\"")
            if Line != '':
                # Make sure ithe # sign is only at the very beggining
                Line = self.rc.rm_spaces(Line)
                # If there is comment sign in the front - it is inactive
                if Line.find('#') == 0:
                    self.FixCheck.set_active(False)
                else:
                    self.FixCheck.set_active(True)
            else:
                self.FixCheck.set_active(False)
                
        self.Reverting = False
            

    def main(self):
        if self.NoFile:
            print "Error: No rc file specified or rc file not compatiable. Aborting ..."
            return 1

        self.window.show()
        gtk.main()
        
        
    def ask_save(self):
        if self.HasChanges:
            dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_QUESTION, buttons= gtk.BUTTONS_YES_NO, message_format= "There are changes made to the file. Would you like to save them?")
            dlg.set_title("Save Changes")
            response = dlg.run()
            dlg.destroy()
            if response == gtk.RESPONSE_YES:
                if not self.save_file():
                    errdlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons= gtk.BUTTONS_OK, message_format= "Save failed. There have to be a problem with the file missing or renamed.")
                    errdlg.set_title("Save Error")                    
                    errdlg.run()
                    errdlg.destroy()
            elif response == gtk.RESPONSE_NO:
                if self.Debug:
                    print "Skipped save"

        return True 
        
        
    def ask_save_stay(self):
        if self.HasChanges:
            dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_QUESTION, buttons= gtk.BUTTONS_OK_CANCEL, message_format= "There are changes made to the file. Would you like to save them and quit or stay?")
            dlg.set_title("Save Changes")
            response = dlg.run()
            dlg.destroy()
            if response == gtk.RESPONSE_OK:
                if not self.save_file():
                    errdlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons= gtk.BUTTONS_OK, message_format= "Save failed. There have to be a problem with the file missing or renamed.")
                    errdlg.set_title("Save Error") 
                    errdlg.run()
                    errdlg.destroy()
                    return False
                else:
                    return True
            elif response == gtk.RESPONSE_CANCEL:
                if self.Debug:
                    print "Skipped save"
                return False
        else:
            return True     
        
        
    def save_file(self):
        # No flag for changes is marked because the user have to be able 
        # to revert to the first loaded file.
        if self.rc.get_filename().find("/usr/share/themes") != -1: #using root theme
            # here unlike load_file are used two differenth methods for speedup
            if self.rc.save_root_file():
                print "File saved."
                self.HasChanges = False
                self.HasSaves = True
                return True
            else:
                print "Failed."
                return False
        else:
            if self.rc.save_file():
                print "File saved."
                self.HasChanges = False
                self.HasSaves = True
                return True
            else:
                print "Failed."
                return False


def LaunchPicker():
     #find installed themes
    checkList = []
    userHome = user.home
    
    # Themes in user's home
    checkList.append (userHome + "/.themes/New Wave/")
    
    checkList.append (userHome + "/.themes/New Wave Dark Menus/")
    # add support for future versions
    checkList.append (userHome + "/.themes/New Wave Minimal/")
    checkList.append (userHome + "/.themes/New Wave Dark Menus Minimal/")
    #check for other theme locations
    checkList.append ("/usr/share/themes/New Wave/")
    checkList.append ("/usr/share/themes/NewWave/")
    checkList.append ("/usr/share/themes/New Wave Dark Menus/")
    checkList.append ("/usr/share/themes/NewWaveDarkMenus/")
    
    dirList = []
    
    
    for path in checkList:
        if os.path.exists(path):
            dirList.append (path)
       
        
    if len(dirList) == 0:
        print "No themes are installed"
        errdlg = gtk.MessageDialog(parent=None, flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons= gtk.BUTTONS_OK, message_format="There are no New Wave themes installed.\nPlease install one or more and try again when ready.")
        errdlg.set_title("New Wave Configurator") 
        errdlg.run()
        errdlg.destroy()
        # if there are no themes return false
        return False
    
    elif len(dirList) > 1:
        #Let the user choose
        #dirList = ["/home/toni/.themes/New Wave", "/usr/share/themes/NewWave"]
        nwd = ThemePickerUI(dirList)
        selected_path = nwd.showdialog()
        
    elif len(dirList) == 1:
        selected_path = dirList[0]

    if selected_path !='':
       
        #Start the ui
        if selected_path.find("Dark") != -1:
            selected_path = selected_path + "gtk-2.0/default-gtkrc"
        else:
            selected_path = selected_path + "gtk-2.0/gtkrc"
        #print "THEME PATH: --->" + selected_path
        
        #Windows dev
        #selected_path = "C:\Documents and Settings\Trud\Desktop\New Wave\gtk-2.0\gtkrc"
        
        nwc = NewWaveConfiguratorUI(selected_path)
        nwc.main()
          
        return True



if __name__ == "__main__":
   
    LaunchPicker()
        


    
