#
# ===== NW Configurator Core v.0.1 =====
#
# This is simple program that configures the
# == New Wave pixmap theme == 
# in an easy way.
# 
# Anton Kerezov <ankere@gmail.com>
# License GPL v3.0

import string
import os.path
import io


class GtkRcFile:
    """A class to handle Gtkrc files """
    mFileContent =  []
    mBackupContent = []
    mHasContent = False
    mNWC_BStart = "#---NWC-START---#" # Block start tag
    mNWC_BEnd = "#---NWC-END---#"     # Block end tag

    def __init__(self, filename=""):
        """ Ctor that inits the class. 
        =================================================================================
        filename = the gtkrc file that would be further processed"""
        self.filename = filename
        # This is the main resource of the class 
        # that contains the data needed to modify 
        # the theme appearance.
        self.mHasContent = False
        
    def load_file(self):
        """Load the file you have specified in the Ctor. 
        =================================================================================
        If load fails False is returned else True.
        """
        root = False
        #Check the filename
        if self.filename == '' :
            return False
        if os.path.exists(self.filename) == False:
            return False
        if self.filename.find("/usr/share/themes/") != -1:
            root = True
        
        #load the file to a string list
        try:
            if root:
                #make the file readable
                os.system("gksu chmod a=rx \"" + self.filename+"\"")
                
            rc_file = open(self.filename, "r")
            self.mFileContent = rc_file.readlines()

            rc_file.close()
            
            if root:
                #bring back the root permissions
                os.system("gksu chown root \"" + self.filename+"\"")
            
            #make a copy of the contents so we could restore if needed            
            self.mBackupContent = self.mFileContent[:]
        
            # Set the flag so that we can save
            self.mHasContent = True
            return True
        except Exception as inst: # handle exceptions here
            print type(inst)     # the exception instance
            print inst.args      # arguments stored in .args
            print inst           # __str__ allows args to printed directly

            print "IO error while trying to open and read the file '", self.filename, "' \n"
            return False
        
    
    def save_file(self):
        """ Save any changes to the file from the Ctor. 
        =================================================================================
        If save fails False is returned else True.
        """
        # We have checked the filename and file contents in load_file
        # so of the flag is True we could proceed normally
        if self.mHasContent == False:
            return False
            
        try:
                
            rc_file = open(self.filename, "w")

            rc_file.writelines(self.mFileContent)
            rc_file.close()

            

            return True
        except:
            print "Could not save file to '", self.filename, "' \n"
            return False

    def save_root_file(self):
        """ Save any changes to the file from the Ctor. 
        =================================================================================
        If save fails False is returned else True. Use this method to 
        save in root password required places.
        """
        # We have checked the filename and file contents in load_file
        # so of the flag is True we could proceed normally
        if self.mHasContent == False:
            return False
        
        try:
            #get the name of the file
            local_file = os.path.basename(self.filename)

            rc_file = open(local_file , "w")
            rc_file.writelines(self.mFileContent)
            rc_file.close()
            
            #copy the file to the actual location with root password asked
            os.system("gksu cp " +local_file+ " \"" +self.filename+ "\"")
            
            #remove the local file
            os.remove(local_file)
            
            return True 
        except:
            print "Could not save file to '", self.filename, "' \n"
            return False
        
    def get_filename(self):
        """ Returns the filename with which the class is initialised. 
        """
        return self.filename
    
        
    def restore_file_data(self):
        """ Restores the file contents to it's original state of first loading.
        """
        self.mFileContent = self.mBackupContent[:]
    
    
    def modify_first_line(self, aOriginal= None, aModified= None):
        """For a given original line find and replace with the modified version.
        ===============================================================================
        aOriginal = The original line to be looked for.
        aModified = The line that would replace aOriginal if match is found in the file
        The function exits after a match is found and replaced.
        """
        if aOriginal == None and aModified == None:
            return False
        else:
            i = 0
            aOriginal = self.__strip_internal(aOriginal)
            in_nwc_block = False #Wether the reading is in nwc block
            
            for line in self.mFileContent:
                #remove unwanted spaces
                line = self.__strip_internal(line)
                
                #check for tags
                if line == self.mNWC_BStart:
                    in_nwc_block = True
                elif line == self.mNWC_BEnd:
                    in_nwc_block = False
                
                #remove comments
                line = self.__remove_comments(line)
                # Only replace if the found match is in editable block    
                if (line == aOriginal) and (in_nwc_block == True):
                    self.mFileContent[i] = aModified + "\n"
                    return True
                
                i = i+1


    def modify_line(self, aOriginal= None, aModified= None):
        """For a given original line find and replace with the modified version.
        ===============================================================================
        aOriginal = The original line to be looked for.
        aModified = The line that would replace aOriginal if match is found in the file
        """
        if aOriginal == None and aModified == None:
            return False
        else:
            i = 0
            aOriginal = self.__strip_internal(aOriginal)
            in_nwc_block = False #Wether the reading is in nwc block
            
            
            for line in self.mFileContent:
                #remove unwanted spaces
                line = self.__strip_internal(line)
                
                
                #check for block tags
                if line == self.mNWC_BStart:
                    in_nwc_block = True
                if line == self.mNWC_BEnd:
                    in_nwc_block = False
                
                line = self.__remove_comments(line)
                # Only replace if the found match is in editable block 
                if (line == aOriginal) and (in_nwc_block == True):
                    self.mFileContent[i] = aModified + "\n"
                    
                i = i+1
                
            return True
    

    def comment_line(self, aOriginal, aCommentOut):
        """Comments out the line in aOriginal according to aCommentOut flag. 
        =================================================================================
        If aCommentOut = true then the line will be commented if it is not and vice versa.
        """
        if aOriginal == None:
            return False
        else:
            i = 0
            #copy is used to presevre spaces and tabs
            copyOriginal = aOriginal
            aOriginal = self.__strip_internal(aOriginal)
            in_nwc_block = False #Wether the reading is in nwc block
            
            
            for line in self.mFileContent:
                #remove unwanted spaces
                line = self.__strip_internal(line)
                
                #check for block tags
                if line == self.mNWC_BStart:
                    in_nwc_block = True
                if line == self.mNWC_BEnd:
                    in_nwc_block = False
                
                # Only replace if the found match is in editable block 
                if (line == aOriginal) and (in_nwc_block == True):
                    if line.find("#") == 0 and aCommentOut == True:
                        #Line already a comment
                        Modified = copyOriginal
                    #this is not the best check as at the end of l
                    #ine there could be other comment
                    elif line.find("#") == 0 and aCommentOut == False:
                        #re,ove any unwanted spaces
                        copyOriginal = copyOriginal.strip()
                        #remove the comment
                        Modified = copyOriginal.lstrip('#')
                    elif line.find("#") != 0 and aCommentOut == True:
                        #add comment
                        Modified = "#" + copyOriginal
                    elif line.find("#") != 0 and aCommentOut == False:
                        #add comment
                        Modified = copyOriginal
                    else: #preserve the original
                        Modified = copyOriginal
    
                    if Modified.find("\n") != -1:
                        self.mFileContent[i] = Modified
                    else:
                        self.mFileContent[i] = Modified + "\n"

                i = i+1
                
            return True

    def find_line(self, containded_text):
        """Search for a given line or phrase in the contents of the 
        gtkrc file and return it as a result.
        =================================================================================
        Empty string means no match is found.
        """
        line = self.__lookup_line(containded_text)
        return line
    
        
    def theme_version(self, v_str):
        """Check the version of the file loaded. 
        =================================================================================
        Use only major versions 0.8 or 0.9 to specify a range of whole release or 
        0.8.1, 0.9.3 for specific ones. You may even use 0. or 1. as v_str
        """
        ver = self.__lookup_line(v_str)
        if ver != '':
            return True
        else:
            return False
        
        
    def get_version_string(self, v_str):
        """Gets the full string specified in the gtkrc file for version. 
        If nothing is found null(empty string) is returned.
        """
        ver = self.__lookup_line(v_str)
        return ver.strip("#")


    def get_property_value(self, property):
        """ Gets the value of the specified property. 
        =================================================================================
        The property name and the given string should match. If the returned string 
        is empty no match is found.
        """
        if property == '':
            return ""
        else:
            line = self.__lookup_line(property)
            if line != '':
                list = line.split('=', 1)
                
                #check if there is comment at the back and remove it too
                if list[1].find('#') != -1:
                    list = list[1].split('#',1)
                    value = list[0]
                else:
                    value = list[1]
                    
                return self.__strip(value)
            else: #No match
                return ""
            
            
    def get_property_value_noNWC(self, property):
        """ Gets a the value of the specified property.
        ================================================================================= 
        The property name and the given string should match. If the returned string 
        is empty no match is found.
        This method is not limited by the NWC tags block tags.
        """
        if property == '':
            return ""
        else:
            line = self.__lookup_line_noNWC(property)
            if line != '':
                list = line.split('=', 1)
                
                #check if there is comment at the back and remove it too
                if list[1].find('#') != -1:
                    list = list[1].split('#',1)
                    value = list[0]
                else:
                    value = list[1]
                    
                return self.__strip(value)
            else: #No match
                return ""
            

    def print_contents(self):
        """Prints the whole document that is loaded to the terminal"""
       # print "\n\nLenght:", len(self.mFileContent), "\n\n"
        print self.mFileContent


    def rm_spaces(self,aLine):
        """ Remove all spaces in the string specified.
        ================================================================================= 
        aLine = the phrase to look for
        """
        return self.__strip_internal(aLine)


    def __strip_internal(self, line):
        """removes unwanetd tabs whitespaces and new lines"""
        line = line.replace(' ','')
        line = line.replace('\t','')
        line = line.replace('\n','')
        return line
    
    
    def __strip(self, line):
        """removes only spaces from both sides of the string"""
        line = line.strip()
        line = line.strip('\t')
        line = line.strip('\n')
        return line


    def __lookup_line(self, aContainingText):
        if aContainingText == '' or aContainingText == None:
            return ''
        else:
            in_nwc_block = False #Wether the reading is in nwc block
            
            for line in self.mFileContent:
                stline = self.__strip(line)
                #check for block tags
                if stline == self.mNWC_BStart:
                    in_nwc_block = True
                if stline == self.mNWC_BEnd:
                    in_nwc_block = False
                    
                if (line.find(aContainingText) != -1) and (in_nwc_block == True):
                    #return the desired line
                    return line
                
                
        return ''
    
    
    def __lookup_line_noNWC(self, aContainingText):
        """ this method is not limited by the NWC tags"""
        if aContainingText == '' or aContainingText == None:
            return ''
        else:
            for line in self.mFileContent:
                if line.find(aContainingText) != -1:
                    #return the desired line
                    return line
        return ''
    
    
    def __remove_comments(self, line):
        if line != '':
            list = line.split('#', 1)
            if len(list) > 1:
                return list[0]
            else:
                return line
        else:
            return line


"""
if __name__ == "__main__":

    rc = GtkRcFile("/home/toni/.themes/New Wave/gtk-2.0/gtkrc")
    
    if rc.load_file():
        #rc.print_contents()
        
        #check the version of New Wave
        if not rc.theme_version("0.8.1"):
            print "Theme not compatiable with configurator."
        else:
            print "Theme compatiable with configurator."
             
         
            #modify properties
            if rc.modify_first_line("GtkScrollbar::    has-backward-stepper= 1", "\tGtkScrollbar   ::has-backward-stepper = 0\n"):
                print "Line modified"
            if rc.modify_first_line("GtkScrollbar::has-secondary-backward-stepper=0", "\tGtkScrollbar   ::has-secondary-backward-stepper = 1\n"):
                print "Line modified"  
            if rc.save_file():
                print "File saved"
          
            print "\n Done."
         
            print rc.get_property_value("base[SELECTED]")
            
    else:
        print "Could not find file"

"""    
